import argparse
import datetime
import logging
import random
import calendar
import copy

from collections import OrderedDict

from dateutil.rrule import rrule, WEEKLY, MO

from .people import People
from .calendarius_utils import fmt_day, _date, logger, weekday_name


def calendarius(start, end, people):
    logger.debug(f"start date: {start}")
    logger.debug(f"end date: {end}")
    logger.debug(f"people: {', '.join(people)}")

    # shuffle people
    # random.shuffle(people)
    logger.debug(f"people: {', '.join(people)}")

    p = People(people)

    if start.weekday() == 0:
        monday = start
    else:
        monday = start + datetime.timedelta(days=-7)

    d = OrderedDict()
    for date in rrule(WEEKLY, byweekday=MO, dtstart=monday, until=end):

        d[date.date()] = p.get_monday()

        logger.debug(f"date: {date.date()} {weekday_name(date)}")

        for i in range(1, 5):
            day = date + datetime.timedelta(days=i)
            d[day.date()] = p.get()

            logger.debug(f"date: {day.date()} {weekday_name(day)}")

    for i in d:
        if i < start.date():
            continue
        print(f"{i} {weekday_name(i)[:3]} : {', '.join(d[i])}")


def main():

    parser = argparse.ArgumentParser(
        prog="calendarius",
        description=__doc__,
        formatter_class=argparse.RawTextHelpFormatter,
    )

    parser.add_argument("--version", action="version", version="0.1.1", help="version")

    parser.add_argument(
        "-v", "--verbose", action="count", default=0, help="increase verbosity"
    )

    parser.add_argument(
        "-s",
        "--start-date",
        type=_date,
        default=datetime.datetime(1970, 1, 1).strftime(fmt_day),
        metavar="YYYY-MM-DD",
        help="start date YYYY-MM-DD",
    )

    parser.add_argument(
        "-e",
        "--end-date",
        type=_date,
        default=datetime.datetime.now().strftime(fmt_day),
        metavar="YYYY-MM-DD",
        help="end date YYYY-MM-DD",
    )

    # Positional parameters
    parser.add_argument("people", default=[], nargs="+", help="list of people")

    args = parser.parse_args()

    if args.verbose:
        logger.setLevel(logging.DEBUG)

    logger.debug(args)

    calendarius(args.start_date, args.end_date, args.people)


if __name__ == "__main__":
    main()



import logging
import datetime
import calendar

fmt_day = "%Y-%m-%d"


def _date(s):
    try:
        return datetime.datetime.strptime(s, fmt_day)
    except:
        raise ValueError("invalid date {}".format(s))


def get_logger(name, verbose=False):
    logging.basicConfig(
        level=logging.INFO,
        format="[%(asctime)s] %(levelname)s - %(message)s",
        datefmt="%H:%M:%S",
    )
    logger = logging.getLogger(name)
    if verbose:
        logger.setLevel(logging.DEBUG)
    return logger


logger = get_logger(__name__)


def weekday_name(day):
    return calendar.day_name[day.weekday()]



import copy
import random

from .calendarius_utils import logger


class People:
    def __init__(self, people):
        self.num_mondays = 0
        self.people = copy.deepcopy(people)
        # check on number of people
        if len(self.people) < 4:
            raise ValueError("ERROR list of people must contain at least 4 persons")

    def get_monday(self):
        """Returns 2 persons for Mondays"""

        # select 2 persons for monday
        self.p1 = self.people[self.num_mondays % len(self.people)]
        self.p2 = self.people[(self.num_mondays + 1) % len(self.people)]

        logger.debug(f"monday selection: {', '.join((self.p1, self.p2))}")

        # copy list of people
        self._work = copy.deepcopy(self.people)

        # remove 2 persons from complete list
        self._work.remove(self.p1)
        self._work.remove(self.p2)

        self.num_mondays += 2

        return self.p1, self.p2

    def get(self):
        """Returns 2 persons from the list excluding people assigned to Monday"""

        p1 = self._select()

        self._remove(p1)

        logger.debug(f"select: {p1}")

        p2 = self._select()

        while p2 == p1:
            p2 = self._select()

        self._remove(p2)

        logger.debug(f"select: {p2}")

        return p1, p2

    def _select(self):
        return random.choice(self._work)

    def _remove(self, p):
        self._work.remove(p)
        # reset list if empty
        if len(self._work) == 0:
            self._reset()

    def _reset(self):
        self._work = copy.deepcopy(self.people)
        self._work.remove(self.p1)
        self._work.remove(self.p2)



from setuptools import setup, find_packages

setup(
    name="calendariUS",
    version="0.1.1",
    author="so07",
    author_email="s.orlandini@cineca.it",
    description="Generate calendar for Help Desk shifts",
    long_description="""Generate calendar for Help Desk shifts""",
    url="",
    packages=find_packages("src"),
    package_dir={"": "src"},
    install_requires=["python-dateutil"],
    entry_points={
        "console_scripts": ["calendarius=calendarius.calendarius:main"],
    },
)
